@@ -39,13 +39,23 @@ class auth_plugin_cas extends auth_plugin_ldap {
     /**
      * Constructor.
      */
-    function auth_plugin_cas() {
+    public function __construct() {
         $this->authtype = 'cas';
         $this->roleauth = 'auth_cas';
         $this->errorlogtag = '[AUTH CAS] ';
         $this->init_plugin($this->authtype);
     }
 
+    /**
+     * Old syntax of class constructor. Deprecated in PHP7.
+     *
+     * @deprecated since Moodle 3.1
+     */
+    public function auth_plugin_cas() {
+        debugging('Use of class name as constructor is deprecated', DEBUG_DEVELOPER);
+        self::__construct();
+    }
+
     function prevent_local_passwords() {
         return true;
     }
@@ -111,6 +121,21 @@ class auth_plugin_cas extends auth_plugin_ldap {
             return;
         }
 
+        // If the multi-authentication setting is used, check for the param before connecting to CAS.
+        if ($this->config->multiauth) {
+
+            // If there is an authentication error, stay on the default authentication page.
+            if (!empty($SESSION->loginerrormsg)) {
+                return;
+            }
+
+            $authCAS = optional_param('authCAS', '', PARAM_RAW);
+            if ($authCAS != 'CAS') {
+                return;
+            }
+
+        }
+
         // Connection to CAS server
         $this->connectCAS();
 
@@ -118,6 +143,7 @@ class auth_plugin_cas extends auth_plugin_ldap {
             $frm = new stdClass();
             $frm->username = phpCAS::getUser();
             $frm->password = 'passwdCas';
+            $frm->logintoken = \core\session\manager::get_login_token();
 
             // Redirect to a course if multi-auth is activated, authCAS is set to CAS and the courseid is specified.
             if ($this->config->multiauth && !empty($courseid)) {
@@ -131,30 +157,10 @@ class auth_plugin_cas extends auth_plugin_ldap {
             $frm = new stdClass();
             $frm->username = 'guest';
             $frm->password = 'guest';
+            $frm->logintoken = \core\session\manager::get_login_token();
             return;
         }
 
-        if ($this->config->multiauth) {
-            $authCAS = optional_param('authCAS', '', PARAM_RAW);
-            if ($authCAS == 'NOCAS') {
-                return;
-            }
-
-            // Show authentication form for multi-authentication
-            // test pgtIou parameter for proxy mode (https connection
-            // in background from CAS server to the php server)
-            if ($authCAS != 'CAS' && !isset($_GET['pgtIou'])) {
-                $PAGE->set_url('/login/index.php');
-                $PAGE->navbar->add($CASform);
-                $PAGE->set_title("$site->fullname: $CASform");
-                $PAGE->set_heading($site->fullname);
-                echo $OUTPUT->header();
-                include($CFG->dirroot.'/auth/cas/cas_form.html');
-                echo $OUTPUT->footer();
-                exit();
-            }
-        }
-
         // Force CAS authentication (if needed).
         if (!phpCAS::isAuthenticated()) {
             phpCAS::setLang($this->config->language);
@@ -162,19 +168,6 @@ class auth_plugin_cas extends auth_plugin_ldap {
         }
     }
 
-    /**
-     * Logout from the CAS
-     *
-     */
-    function prelogout_hook() {
-        global $CFG;
-
-        if (!empty($this->config->logoutcas)) {
-            $backurl = $CFG->wwwroot;
-            $this->connectCAS();
-            phpCAS::logoutWithURL($backurl);
-        }
-    }
 
     /**
      * Connect to the CAS (clientcas connection or proxycas connection)
@@ -191,11 +184,16 @@ class auth_plugin_cas extends auth_plugin_ldap {
             } else {
                 phpCAS::client($this->config->casversion, $this->config->hostname, (int) $this->config->port, $this->config->baseuri, false);
             }
+            // Some CAS installs require SSLv3 that should be explicitly set.
+            if (!empty($this->config->curl_ssl_version)) {
+                phpCAS::setExtraCurlOption(CURLOPT_SSLVERSION, $this->config->curl_ssl_version);
+            }
+
             $connected = true;
         }
 
         // If Moodle is configured to use a proxy, phpCAS needs some curl options set.
-        if (!empty($CFG->proxyhost) && !is_proxybypass($this->config->hostname)) {
+        if (!empty($CFG->proxyhost) && !is_proxybypass(phpCAS::getServerLoginURL())) {
             phpCAS::setExtraCurlOption(CURLOPT_PROXY, $CFG->proxyhost);
             if (!empty($CFG->proxyport)) {
                 phpCAS::setExtraCurlOption(CURLOPT_PROXYPORT, $CFG->proxyport);
@@ -223,47 +221,6 @@ class auth_plugin_cas extends auth_plugin_ldap {
         }
     }
 
-    /**
-     * Prints a form for configuring this authentication plugin.
-     *
-     * This function is called from admin/auth.php, and outputs a full page with
-     * a form for configuring this plugin.
-     *
-     * @param array $page An object containing all the data for this page.
-     */
-    function config_form($config, $err, $user_fields) {
-        global $CFG, $OUTPUT;
-
-        if (!function_exists('ldap_connect')) { // Is php-ldap really there?
-            echo $OUTPUT->notification(get_string('auth_ldap_noextension', 'auth_ldap'));
-
-            // Don't return here, like we do in auth/ldap. We cas use CAS without LDAP.
-            // So just warn the user (done above) and define the LDAP constants we use
-            // in config.html, to silence the warnings.
-            if (!defined('LDAP_DEREF_NEVER')) {
-                define ('LDAP_DEREF_NEVER', 0);
-            }
-            if (!defined('LDAP_DEREF_ALWAYS')) {
-                define ('LDAP_DEREF_ALWAYS', 3);
-            }
-        }
-
-        include($CFG->dirroot.'/auth/cas/config.html');
-    }
-
-    /**
-     * A chance to validate form data, and last chance to
-     * do stuff before it is inserted in config_plugin
-     * @param object object with submitted configuration settings (without system magic quotes)
-     * @param array $err array of error messages
-     */
-    function validate_form($form, &$err) {
-        $certificate_path = trim($form->certificate_path);
-        if ($form->certificate_check && empty($certificate_path)) {
-            $err['certificate_path'] = get_string('auth_cas_certificate_path_empty', 'auth_cas');
-        }
-    }
-
     /**
      * Returns the URL for changing the user's pw, or empty if the default can
      * be used.
@@ -274,139 +231,6 @@ class auth_plugin_cas extends auth_plugin_ldap {
         return null;
     }
 
-    /**
-     * Processes and stores configuration data for this authentication plugin.
-     */
-    function process_config($config) {
-
-        // CAS settings
-        if (!isset($config->hostname)) {
-            $config->hostname = '';
-        }
-        if (!isset($config->port)) {
-            $config->port = '';
-        }
-        if (!isset($config->casversion)) {
-            $config->casversion = '';
-        }
-        if (!isset($config->baseuri)) {
-            $config->baseuri = '';
-        }
-        if (!isset($config->language)) {
-            $config->language = '';
-        }
-        if (!isset($config->proxycas)) {
-            $config->proxycas = '';
-        }
-        if (!isset($config->logoutcas)) {
-            $config->logoutcas = '';
-        }
-        if (!isset($config->multiauth)) {
-            $config->multiauth = '';
-        }
-        if (!isset($config->certificate_check)) {
-            $config->certificate_check = '';
-        }
-        if (!isset($config->certificate_path)) {
-            $config->certificate_path = '';
-        }
-        if (!isset($config->logout_return_url)) {
-            $config->logout_return_url = '';
-        }
-
-        // LDAP settings
-        if (!isset($config->host_url)) {
-            $config->host_url = '';
-        }
-        if (!isset($config->start_tls)) {
-             $config->start_tls = false;
-        }
-        if (empty($config->ldapencoding)) {
-            $config->ldapencoding = 'utf-8';
-        }
-        if (!isset($config->pagesize)) {
-            $config->pagesize = LDAP_DEFAULT_PAGESIZE;
-        }
-        if (!isset($config->contexts)) {
-            $config->contexts = '';
-        }
-        if (!isset($config->user_type)) {
-            $config->user_type = 'default';
-        }
-        if (!isset($config->user_attribute)) {
-            $config->user_attribute = '';
-        }
-        if (!isset($config->search_sub)) {
-            $config->search_sub = '';
-        }
-        if (!isset($config->opt_deref)) {
-            $config->opt_deref = LDAP_DEREF_NEVER;
-        }
-        if (!isset($config->bind_dn)) {
-            $config->bind_dn = '';
-        }
-        if (!isset($config->bind_pw)) {
-            $config->bind_pw = '';
-        }
-        if (!isset($config->ldap_version)) {
-            $config->ldap_version = '3';
-        }
-        if (!isset($config->objectclass)) {
-            $config->objectclass = '';
-        }
-        if (!isset($config->memberattribute)) {
-            $config->memberattribute = '';
-        }
-
-        if (!isset($config->memberattribute_isdn)) {
-            $config->memberattribute_isdn = '';
-        }
-        if (!isset($config->attrcreators)) {
-            $config->attrcreators = '';
-        }
-        if (!isset($config->groupecreators)) {
-            $config->groupecreators = '';
-        }
-        if (!isset($config->removeuser)) {
-            $config->removeuser = AUTH_REMOVEUSER_KEEP;
-        }
-
-        // save CAS settings
-        set_config('hostname', trim($config->hostname), $this->pluginconfig);
-        set_config('port', trim($config->port), $this->pluginconfig);
-        set_config('casversion', $config->casversion, $this->pluginconfig);
-        set_config('baseuri', trim($config->baseuri), $this->pluginconfig);
-        set_config('language', $config->language, $this->pluginconfig);
-        set_config('proxycas', $config->proxycas, $this->pluginconfig);
-        set_config('logoutcas', $config->logoutcas, $this->pluginconfig);
-        set_config('multiauth', $config->multiauth, $this->pluginconfig);
-        set_config('certificate_check', $config->certificate_check, $this->pluginconfig);
-        set_config('certificate_path', $config->certificate_path, $this->pluginconfig);
-        set_config('logout_return_url', $config->logout_return_url, $this->pluginconfig);
-
-        // save LDAP settings
-        set_config('host_url', trim($config->host_url), $this->pluginconfig);
-        set_config('start_tls', $config->start_tls, $this->pluginconfig);
-        set_config('ldapencoding', trim($config->ldapencoding), $this->pluginconfig);
-        set_config('pagesize', (int)trim($config->pagesize), $this->pluginconfig);
-        set_config('contexts', trim($config->contexts), $this->pluginconfig);
-        set_config('user_type', core_text::strtolower(trim($config->user_type)), $this->pluginconfig);
-        set_config('user_attribute', core_text::strtolower(trim($config->user_attribute)), $this->pluginconfig);
-        set_config('search_sub', $config->search_sub, $this->pluginconfig);
-        set_config('opt_deref', $config->opt_deref, $this->pluginconfig);
-        set_config('bind_dn', trim($config->bind_dn), $this->pluginconfig);
-        set_config('bind_pw', $config->bind_pw, $this->pluginconfig);
-        set_config('ldap_version', $config->ldap_version, $this->pluginconfig);
-        set_config('objectclass', trim($config->objectclass), $this->pluginconfig);
-        set_config('memberattribute', core_text::strtolower(trim($config->memberattribute)), $this->pluginconfig);
-        set_config('memberattribute_isdn', $config->memberattribute_isdn, $this->pluginconfig);
-        set_config('attrcreators', trim($config->attrcreators), $this->pluginconfig);
-        set_config('groupecreators', trim($config->groupecreators), $this->pluginconfig);
-        set_config('removeuser', $config->removeuser, $this->pluginconfig);
-
-        return true;
-    }
-
     /**
      * Returns true if user should be coursecreator.
      *
@@ -510,4 +334,53 @@ class auth_plugin_cas extends auth_plugin_ldap {
             }
         }
     }
+
+    /**
+     * Post logout hook.
+     *
+     * Note: this method replace the prelogout_hook method to avoid redirect to CAS logout
+     * before the event userlogout being triggered.
+     *
+     * @param stdClass $user clone of USER object object before the user session was terminated
+     */
+    public function postlogout_hook($user) {
+        global $CFG;
+        // Only redirect to CAS logout if the user is logged as a CAS user.
+        if (!empty($this->config->logoutcas) && $user->auth == $this->authtype) {
+            $backurl = !empty($this->config->logout_return_url) ? $this->config->logout_return_url : $CFG->wwwroot;
+            $this->connectCAS();
+            phpCAS::logoutWithRedirectService($backurl);
+        }
+    }
+
+    /**
+     * Return a list of identity providers to display on the login page.
+     *
+     * @param string|moodle_url $wantsurl The requested URL.
+     * @return array List of arrays with keys url, iconurl and name.
+     */
+    public function loginpage_idp_list($wantsurl) {
+        if (empty($this->config->hostname)) {
+            // CAS is not configured.
+            return [];
+        }
+
+        $iconurl = moodle_url::make_pluginfile_url(
+            context_system::instance()->id,
+            'auth_cas',
+            'logo',
+            null,
+            '/',
+            $this->config->auth_logo);
+
+        return [
+            [
+                'url' => new moodle_url(get_login_url(), [
+                        'authCAS' => 'CAS',
+                    ]),
+                'iconurl' => $iconurl,
+                'name' => format_string($this->config->auth_name),
+            ],
+        ];
+    }
 }
