@@ -412,22 +412,24 @@ class behat_general extends behat_base {
      * @param string $containerelement
      * @param string $containerselectortype
      */
-    public function i_drag_and_i_drop_it_in($source, $sourcetype, $target, $targettype) {
-        if (!$this->running_javascript()) {
-            throw new DriverException('Drag and drop steps require javascript');
+    public function i_drag_and_i_drop_it_in($element, $selectortype, $containerelement, $containerselectortype) {
+
+        list($sourceselector, $sourcelocator) = $this->transform_selector($selectortype, $element);
+        $sourcexpath = $this->getSession()->getSelectorsHandler()->selectorToXpath($sourceselector, $sourcelocator);
+
+        list($containerselector, $containerlocator) = $this->transform_selector($containerselectortype, $containerelement);
+        $destinationxpath = $this->getSession()->getSelectorsHandler()->selectorToXpath($containerselector, $containerlocator);
+
+        $node = $this->get_selected_node("xpath_element", $sourcexpath);
+        if (!$node->isVisible()) {
+            throw new ExpectationException('"' . $sourcexpath . '" "xpath_element" is not visible', $this->getSession());
+        }
+        $node = $this->get_selected_node("xpath_element", $destinationxpath);
+        if (!$node->isVisible()) {
+            throw new ExpectationException('"' . $destinationxpath . '" "xpath_element" is not visible', $this->getSession());
         }
 
-        $source = $this->find($sourcetype, $source);
-        $target = $this->find($targettype, $target);
-
-        if (!$source->isVisible()) {
-            throw new ExpectationException("'{$source}' '{$sourcetype}' is not visible", $this->getSession());
-        }
-        if (!$target->isVisible()) {
-            throw new ExpectationException("'{$target}' '{$targettype}' is not visible", $this->getSession());
-        }
-
-        $this->getSession()->getDriver()->dragTo($source->getXpath(), $target->getXpath());
+        $this->getSession()->getDriver()->dragTo($sourcexpath, $destinationxpath);
     }
 
     /**
@@ -761,126 +763,53 @@ class behat_general extends behat_base {
     /**
      * Checks, that the first specified element appears before the second one.
      *
-     * @Then :preelement :preselectortype should appear before :postelement :postselectortype
-     * @Then :preelement :preselectortype should appear before :postelement :postselectortype in the :containerelement :containerselectortype
+     * @Given /^"(?P<preceding_element_string>(?:[^"]|\\")*)" "(?P<selector1_string>(?:[^"]|\\")*)" should appear before "(?P<following_element_string>(?:[^"]|\\")*)" "(?P<selector2_string>(?:[^"]|\\")*)"$/
      * @throws ExpectationException
      * @param string $preelement The locator of the preceding element
-     * @param string $preselectortype The selector type of the preceding element
+     * @param string $preselectortype The locator of the preceding element
      * @param string $postelement The locator of the latest element
      * @param string $postselectortype The selector type of the latest element
-     * @param string $containerelement
-     * @param string $containerselectortype
      */
-    public function should_appear_before(
-        string $preelement,
-        string $preselectortype,
-        string $postelement,
-        string $postselectortype,
-        $containerelement = null,
-        $containerselectortype = null
-    ) {
-        $msg = "'{$preelement}' '{$preselectortype}' does not appear before '{$postelement}' '{$postselectortype}'";
-        $this->check_element_order(
-            $containerelement,
-            $containerselectortype,
-            $preelement,
-            $preselectortype,
-            $postelement,
-            $postselectortype,
-            $msg
-        );
+    public function should_appear_before($preelement, $preselectortype, $postelement, $postselectortype) {
+
+        // We allow postselectortype as a non-text based selector.
+        list($preselector, $prelocator) = $this->transform_selector($preselectortype, $preelement);
+        list($postselector, $postlocator) = $this->transform_selector($postselectortype, $postelement);
+
+        $prexpath = $this->find($preselector, $prelocator)->getXpath();
+        $postxpath = $this->find($postselector, $postlocator)->getXpath();
+
+        // Using following xpath axe to find it.
+        $msg = '"'.$preelement.'" "'.$preselectortype.'" does not appear before "'.$postelement.'" "'.$postselectortype.'"';
+        $xpath = $prexpath.'/following::*[contains(., '.$postxpath.')]';
+        if (!$this->getSession()->getDriver()->find($xpath)) {
+            throw new ExpectationException($msg, $this->getSession());
+        }
     }
 
     /**
      * Checks, that the first specified element appears after the second one.
      *
-     * @Then :postelement :postselectortype should appear after :preelement :preselectortype
-     * @Then :postelement :postselectortype should appear after :preelement :preselectortype in the :containerelement :containerselectortype
+     * @Given /^"(?P<following_element_string>(?:[^"]|\\")*)" "(?P<selector1_string>(?:[^"]|\\")*)" should appear after "(?P<preceding_element_string>(?:[^"]|\\")*)" "(?P<selector2_string>(?:[^"]|\\")*)"$/
      * @throws ExpectationException
      * @param string $postelement The locator of the latest element
      * @param string $postselectortype The selector type of the latest element
      * @param string $preelement The locator of the preceding element
-     * @param string $preselectortype The selector type of the preceding element
-     * @param string $containerelement
-     * @param string $containerselectortype
-     */
-    public function should_appear_after(
-        string $postelement,
-        string $postselectortype,
-        string $preelement,
-        string $preselectortype,
-        $containerelement = null,
-        $containerselectortype = null
-    ) {
-        $msg = "'{$postelement}' '{$postselectortype}' does not appear after '{$preelement}' '{$preselectortype}'";
-        $this->check_element_order(
-            $containerelement,
-            $containerselectortype,
-            $preelement,
-            $preselectortype,
-            $postelement,
-            $postselectortype,
-            $msg
-        );
-    }
-
-    /**
-     * Shared code to check whether an element is before or after another one.
-     *
-     * @param string $containerelement
-     * @param string $containerselectortype
-     * @param string $preelement The locator of the preceding element
      * @param string $preselectortype The locator of the preceding element
-     * @param string $postelement The locator of the following element
-     * @param string $postselectortype The selector type of the following element
-     * @param string $msg Message to output if this fails
      */
-    protected function check_element_order(
-        $containerelement,
-        $containerselectortype,
-        string $preelement,
-        string $preselectortype,
-        string $postelement,
-        string $postselectortype,
-        string $msg
-    ) {
-        $containernode = false;
-        if ($containerselectortype && $containerelement) {
-            // Get the container node.
-            $containernode = $this->get_selected_node($containerselectortype, $containerelement);
-            $msg .= " in the '{$containerelement}' '{$containerselectortype}'";
-        }
+    public function should_appear_after($postelement, $postselectortype, $preelement, $preselectortype) {
 
-        list($preselector, $prelocator) = $this->transform_selector($preselectortype, $preelement);
+        // We allow postselectortype as a non-text based selector.
         list($postselector, $postlocator) = $this->transform_selector($postselectortype, $postelement);
+        list($preselector, $prelocator) = $this->transform_selector($preselectortype, $preelement);
 
-        $newlines = [
-            "\r\n",
-            "\r",
-            "\n",
-        ];
-        $prexpath = str_replace($newlines, ' ', $this->find($preselector, $prelocator, false, $containernode)->getXpath());
-        $postxpath = str_replace($newlines, ' ', $this->find($postselector, $postlocator, false, $containernode)->getXpath());
+        $postxpath = $this->find($postselector, $postlocator)->getXpath();
+        $prexpath = $this->find($preselector, $prelocator)->getXpath();
 
-        if ($this->running_javascript()) {
-            // The xpath to do this was running really slowly on certain Chrome versions so we are using
-            // this DOM method instead.
-            $js = <<<EOF
-(function() {
-    var a = document.evaluate("{$prexpath}", document, null, XPathResult.ANY_TYPE, null).iterateNext();
-    var b = document.evaluate("{$postxpath}", document, null, XPathResult.ANY_TYPE, null).iterateNext();
-    return a.compareDocumentPosition(b) & Node.DOCUMENT_POSITION_FOLLOWING;
-})()
-EOF;
-            $ok = $this->getSession()->getDriver()->evaluateScript($js);
-        } else {
-
-            // Using following xpath axe to find it.
-            $xpath = "{$prexpath}/following::*[contains(., {$postxpath})]";
-            $ok = $this->getSession()->getDriver()->find($xpath);
-        }
-
-        if (!$ok) {
+        // Using preceding xpath axe to find it.
+        $msg = '"'.$postelement.'" "'.$postselectortype.'" does not appear after "'.$preelement.'" "'.$preselectortype.'"';
+        $xpath = $postxpath.'/preceding::*[contains(., '.$prexpath.')]';
+        if (!$this->getSession()->getDriver()->find($xpath)) {
             throw new ExpectationException($msg, $this->getSession());
         }
     }
@@ -966,8 +895,12 @@ EOF;
      * @param string $selectortype The selector type
      */
     public function should_exist($element, $selectortype) {
+
+        // Getting Mink selector and locator.
+        list($selector, $locator) = $this->transform_selector($selectortype, $element);
+
         // Will throw an ElementNotFoundException if it does not exist.
-        $this->find($selectortype, $element);
+        $this->find($selector, $locator);
     }
 
     /**
@@ -981,27 +914,35 @@ EOF;
      * @param string $selectortype The selector type
      */
     public function should_not_exist($element, $selectortype) {
-        // Will throw an ElementNotFoundException if it does not exist, but, actually it should not exist, so we try &
-        // catch it.
+
+        // Getting Mink selector and locator.
+        list($selector, $locator) = $this->transform_selector($selectortype, $element);
+
         try {
+
+            // Using directly the spin method as we want a reduced timeout but there is no
+            // need for a 0.1 seconds interval because in the optimistic case we will timeout.
+            $params = array('selector' => $selector, 'locator' => $locator);
             // The exception does not really matter as we will catch it and will never "explode".
             $exception = new ElementNotFoundException($this->getSession(), $selectortype, null, $element);
 
-            // Using the spin method as we want a reduced timeout but there is no need for a 0.1 seconds interval
-            // because in the optimistic case we will timeout.
             // If all goes good it will throw an ElementNotFoundExceptionn that we will catch.
             $this->spin(
-                function($context, $args) use ($selectortype, $element) {
-                    return $this->find($selectortype, $element);
-                }, [], behat_base::get_reduced_timeout(), $exception, false
+                function($context, $args) {
+                    return $context->getSession()->getPage()->findAll($args['selector'], $args['locator']);
+                },
+                $params,
+                behat_base::get_reduced_timeout(),
+                $exception,
+                false
             );
         } catch (ElementNotFoundException $e) {
-            // We expect the element to not be found.
+            // It passes.
             return;
         }
 
-        // The element was found and should not have been. Throw an exception.
-        throw new ExpectationException("The '{$element}' '{$selectortype}' exists in the current page", $this->getSession());
+        throw new ExpectationException('The "' . $element . '" "' . $selectortype .
+                '" exists in the current page', $this->getSession());
     }
 
     /**
@@ -1145,14 +1086,16 @@ EOF;
      */
     public function should_exist_in_the($element, $selectortype, $containerelement, $containerselectortype) {
         // Get the container node.
-        $containernode = $this->find($containerselectortype, $containerelement);
+        $containernode = $this->get_selected_node($containerselectortype, $containerelement);
+
+        list($selector, $locator) = $this->transform_selector($selectortype, $element);
 
         // Specific exception giving info about where can't we find the element.
-        $locatorexceptionmsg = "{$element} in the {$containerelement} {$containerselectortype}";
+        $locatorexceptionmsg = $element . '" in the "' . $containerelement. '" "' . $containerselectortype. '"';
         $exception = new ElementNotFoundException($this->getSession(), $selectortype, null, $locatorexceptionmsg);
 
         // Looks for the requested node inside the container node.
-        $this->find($selectortype, $element, $exception, $containernode);
+        $this->find($selector, $locator, $exception, $containernode);
     }
 
     /**
@@ -1168,24 +1111,26 @@ EOF;
      * @param string $containerselectortype The container locator
      */
     public function should_not_exist_in_the($element, $selectortype, $containerelement, $containerselectortype) {
-        // Get the container node.
-        $containernode = $this->find($containerselectortype, $containerelement);
 
-        // Will throw an ElementNotFoundException if it does not exist, but, actually it should not exist, so we try &
-        // catch it.
+        // Get the container node; here we throw an exception
+        // if the container node does not exist.
+        $containernode = $this->get_selected_node($containerselectortype, $containerelement);
+
+        list($selector, $locator) = $this->transform_selector($selectortype, $element);
+
+        // Will throw an ElementNotFoundException if it does not exist, but, actually
+        // it should not exist, so we try & catch it.
         try {
-            // Looks for the requested node inside the container node.
-            $this->find($selectortype, $element, false, $containernode, behat_base::get_reduced_timeout());
+            // Would be better to use a 1 second sleep because the element should not be there,
+            // but we would need to duplicate the whole find_all() logic to do it, the benefit of
+            // changing to 1 second sleep is not significant.
+            $this->find($selector, $locator, false, $containernode, behat_base::get_reduced_timeout());
         } catch (ElementNotFoundException $e) {
-            // We expect the element to not be found.
+            // It passes.
             return;
         }
-
-        // The element was found and should not have been. Throw an exception.
-        throw new ExpectationException(
-            "The '{$element}' '{$selectortype}' exists in the '{$containerelement}' '{$containerselectortype}'",
-            $this->getSession()
-        );
+        throw new ExpectationException('The "' . $element . '" "' . $selectortype . '" exists in the "' .
+                $containerelement . '" "' . $containerselectortype . '"', $this->getSession());
     }
 
     /**
@@ -1197,7 +1142,6 @@ EOF;
      *
      * @throws ExpectationException
      * @Then /^I change (window|viewport) size to "(small|medium|large|\d+x\d+)"$/
-     * @Then /^I change the (window|viewport) size to "(small|medium|large|\d+x\d+)"$/
      * @param string $windowsize size of the window (small|medium|large|wxh).
      */
     public function i_change_window_size_to($windowviewport, $windowsize) {
@@ -1500,6 +1444,46 @@ EOF;
         }
     }
 
+    /**
+     * Checks that the image on the page is the same as one of the fixture files
+     *
+     * @Then /^the image at "(?P<element_string>(?:[^"]|\\")*)" "(?P<selector_string>[^"]*)" should be identical to "(?P<filepath_string>(?:[^"]|\\")*)"$/
+     * @throws ExpectationException
+     * @param string $element The locator of the image
+     * @param string $selectortype The selector type
+     * @param string $filepath path to the fixture file
+     */
+    public function the_image_at_should_be_identical_to($element, $selectortype, $filepath) {
+        global $CFG;
+
+        // Get the container node (exception if it doesn't exist).
+        $containernode = $this->get_selected_node($selectortype, $element);
+        $url = $containernode->getAttribute('src');
+        if ($url == null) {
+            throw new ExpectationException('Element does not have src attribute',
+                $this->getSession());
+        }
+        $session = $this->getSession()->getCookie('MoodleSession');
+        $content = download_file_content($url, array('Cookie' => 'MoodleSession=' . $session));
+
+        // Get the content of the fixture file.
+        // Replace 'admin/' if it is in start of path with $CFG->admin .
+        if (substr($filepath, 0, 6) === 'admin/') {
+            $filepath = $CFG->admin . DIRECTORY_SEPARATOR . substr($filepath, 6);
+        }
+        $filepath = str_replace('/', DIRECTORY_SEPARATOR, $filepath);
+        $filepath = $CFG->dirroot . DIRECTORY_SEPARATOR . $filepath;
+        if (!is_readable($filepath)) {
+            throw new ExpectationException('The file to compare to does not exist.', $this->getSession());
+        }
+        $expectedcontent = file_get_contents($filepath);
+
+        if ($content !== $expectedcontent) {
+            throw new ExpectationException('Image is not identical to the fixture. Received ' .
+            strlen($content) . ' bytes and expected ' . strlen($expectedcontent) . ' bytes');
+        }
+    }
+
     /**
      * Prepare to detect whether or not a new page has loaded (or the same page reloaded) some time in the future.
      *
@@ -1595,9 +1579,27 @@ EOF;
      *
      * @Then /^(?:|I )pause(?:| scenario execution)$/
      */
-    public function i_pause_scenario_execution() {
-        $message = "<colour:lightYellow>Paused. Press <colour:lightRed>Enter/Return<colour:lightYellow> to continue.";
-        behat_util::pause($this->getSession(), $message);
+    public function i_pause_scenario_executon() {
+        global $CFG;
+
+        $posixexists = function_exists('posix_isatty');
+
+        // Make sure this step is only used with interactive terminal (if detected).
+        if ($posixexists && !@posix_isatty(STDOUT)) {
+            $session = $this->getSession();
+            throw new ExpectationException('Break point should only be used with interative terminal.', $session);
+        }
+
+        // Windows don't support ANSI code by default, but with ANSICON.
+        $isansicon = getenv('ANSICON');
+        if (($CFG->ostype === 'WINDOWS') && empty($isansicon)) {
+            fwrite(STDOUT, "Paused. Press Enter/Return to continue.");
+            fread(STDIN, 1024);
+        } else {
+            fwrite(STDOUT, "\033[s\n\033[0;93mPaused. Press \033[1;31mEnter/Return\033[0;93m to continue.\033[0m");
+            fread(STDIN, 1024);
+            fwrite(STDOUT, "\033[2A\033[u\033[2B");
+        }
     }
 
     /**
@@ -1718,8 +1720,8 @@ EOF;
         if (!$this->running_javascript()) {
             throw new DriverException('Checking focus on an element requires JavaScript');
         }
-
-        $element = $this->find($nodeselectortype, $nodeelement);
+        list($a, $b) = $this->transform_selector($nodeselectortype, $nodeelement);
+        $element = $this->find($a, $b);
         $xpath = addslashes_js($element->getXpath());
         $script = 'return (function() { return document.activeElement === document.evaluate("' . $xpath . '",
                 document, null, XPathResult.FIRST_ORDERED_NODE_TYPE, null).singleNodeValue; })(); ';
