@@ -36,6 +36,23 @@ require_once($CFG->libdir . '/tablelib.php');
  */
 class mod_feedback_responses_table extends table_sql {
 
+    /**
+     * Maximum number of feedback questions to display in the "Show responses" table
+     */
+    const PREVIEWCOLUMNSLIMIT = 10;
+
+    /**
+     * Maximum number of feedback questions answers to retrieve in one SQL query.
+     * Mysql has a limit of 60, we leave 1 for joining with users table.
+     */
+    const TABLEJOINLIMIT = 59;
+
+    /**
+     * When additional queries are needed to retrieve more than TABLEJOINLIMIT questions answers, do it in chunks every x rows.
+     * Value too small will mean too many DB queries, value too big may cause memory overflow.
+     */
+    const ROWCHUNKSIZE = 100;
+
     /** @var mod_feedback_structure */
     protected $feedbackstructure;
 
@@ -51,12 +68,23 @@ class mod_feedback_responses_table extends table_sql {
     /** @var string */
     protected $downloadparamname = 'download';
 
+    /** @var int number of columns that were not retrieved in the main SQL query
+     * (no more than TABLEJOINLIMIT tables with values can be joined). */
+    protected $hasmorecolumns = 0;
+
+    /** @var bool whether we are building this table for a external function */
+    protected $buildforexternal = false;
+
+    /** @var array the data structure containing the table data for the external function */
+    protected $dataforexternal = [];
+
     /**
      * Constructor
      *
      * @param mod_feedback_structure $feedbackstructure
+     * @param int $group retrieve only users from this group (optional)
      */
-    public function __construct(mod_feedback_structure $feedbackstructure) {
+    public function __construct(mod_feedback_structure $feedbackstructure, $group = 0) {
         $this->feedbackstructure = $feedbackstructure;
 
         parent::__construct('feedback-showentry-list-' . $feedbackstructure->get_cm()->instance);
@@ -76,16 +104,21 @@ class mod_feedback_responses_table extends table_sql {
         $this->is_downloading(optional_param($this->downloadparamname, 0, PARAM_ALPHA),
                 $name, get_string('responses', 'feedback'));
         $this->useridfield = 'userid';
-        $this->init();
+        $this->init($group);
     }
 
     /**
      * Initialises table
+     * @param int $group retrieve only users from this group (optional)
      */
-    protected function init() {
+    protected function init($group = 0) {
 
-        $tablecolumns = array('userpic', 'fullname');
-        $tableheaders = array(get_string('userpic'), get_string('fullnameuser'));
+        $tablecolumns = array('userpic', 'fullname', 'groups');
+        $tableheaders = array(
+            get_string('userpic'),
+            get_string('fullnameuser'),
+            get_string('groups')
+        );
 
         $extrafields = get_extra_user_fields($this->get_context());
         $ufields = user_picture::fields('u', $extrafields, $this->useridfield);
@@ -133,7 +166,7 @@ class mod_feedback_responses_table extends table_sql {
         $params['notdeleted'] = 0;
         $params['courseid'] = $this->feedbackstructure->get_courseid();
 
-        $group = groups_get_activity_group($this->feedbackstructure->get_cm(), true);
+        $group = (empty($group)) ? groups_get_activity_group($this->feedbackstructure->get_cm(), true) : $group;
         if ($group) {
             $where .= ' AND c.userid IN (SELECT g.userid FROM {groups_members} g WHERE g.groupid = :group)';
             $params['group'] = $group;
@@ -229,6 +262,21 @@ class mod_feedback_responses_table extends table_sql {
         return $name;
     }
 
+    /**
+     * Prepares column groups for display
+     * @param array $row
+     * @return string
+     */
+    public function col_groups($row) {
+        $groups = '';
+        if ($usergrps = groups_get_all_groups($this->feedbackstructure->get_cm()->course, $row->userid, 0, 'name')) {
+            foreach ($usergrps as $group) {
+                $groups .= format_string($group->name). ' ';
+            }
+        }
+        return trim($groups);
+    }
+
     /**
      * Adds common values to the table that do not change the number or order of entries and
      * are only needed when outputting or downloading data.
@@ -237,13 +285,27 @@ class mod_feedback_responses_table extends table_sql {
         $tablecolumns = array_keys($this->columns);
         $tableheaders = $this->headers;
 
-        // Add all feedback response values.
         $items = $this->feedbackstructure->get_items(true);
+        if (!$this->is_downloading() && !$this->buildforexternal) {
+            // In preview mode do not show all columns or the page becomes unreadable.
+            // The information message will be displayed to the teacher that the rest of the data can be viewed when downloading.
+            $items = array_slice($items, 0, self::PREVIEWCOLUMNSLIMIT, true);
+        }
+
+        $columnscount = 0;
+        $this->hasmorecolumns = max(0, count($items) - self::TABLEJOINLIMIT);
+
+        // Add feedback response values.
         foreach ($items as $nr => $item) {
-            $this->sql->fields .= ", v{$nr}.value AS val{$nr}";
-            $this->sql->from .= " LEFT OUTER JOIN {feedback_value} v{$nr} " .
-                "ON v{$nr}.completed = c.id AND v{$nr}.item = :itemid{$nr}";
-            $this->sql->params["itemid{$nr}"] = $item->id;
+            if ($columnscount++ < self::TABLEJOINLIMIT) {
+                // Mysql has a limit on the number of tables in the join, so we only add limited number of columns here,
+                // the rest will be added in {@link self::build_table()} and {@link self::build_table_chunk()} functions.
+                $this->sql->fields .= ", v{$nr}.value AS val{$nr}";
+                $this->sql->from .= " LEFT OUTER JOIN {feedback_value} v{$nr} " .
+                    "ON v{$nr}.completed = c.id AND v{$nr}.item = :itemid{$nr}";
+                $this->sql->params["itemid{$nr}"] = $item->id;
+            }
+
             $tablecolumns[] = "val{$nr}";
             $itemobj = feedback_get_item_class($item->typ);
             $tableheaders[] = $itemobj->get_display_name($item);
@@ -352,6 +414,11 @@ class mod_feedback_responses_table extends table_sql {
             echo $OUTPUT->box(get_string('nothingtodisplay'), 'generalbox nothingtodisplay');
             return;
         }
+
+        if (count($this->feedbackstructure->get_items(true)) > self::PREVIEWCOLUMNSLIMIT) {
+            echo $OUTPUT->notification(get_string('questionslimited', 'feedback', self::PREVIEWCOLUMNSLIMIT), 'info');
+        }
+
         $this->out($this->showall ? $grandtotal : FEEDBACK_DEFAULT_PAGE_COUNT,
                 $grandtotal > FEEDBACK_DEFAULT_PAGE_COUNT);
 
@@ -412,6 +479,95 @@ class mod_feedback_responses_table extends table_sql {
         exit;
     }
 
+    /**
+     * Take the data returned from the db_query and go through all the rows
+     * processing each col using either col_{columnname} method or other_cols
+     * method or if other_cols returns NULL then put the data straight into the
+     * table.
+     *
+     * This overwrites the parent method because full SQL query may fail on Mysql
+     * because of the limit in the number of tables in the join. Therefore we only
+     * join 59 tables in the main query and add the rest here.
+     *
+     * @return void
+     */
+    public function build_table() {
+        if ($this->rawdata instanceof \Traversable && !$this->rawdata->valid()) {
+            return;
+        }
+        if (!$this->rawdata) {
+            return;
+        }
+
+        $columnsgroups = [];
+        if ($this->hasmorecolumns) {
+            $items = $this->feedbackstructure->get_items(true);
+            $notretrieveditems = array_slice($items, self::TABLEJOINLIMIT, $this->hasmorecolumns, true);
+            $columnsgroups = array_chunk($notretrieveditems, self::TABLEJOINLIMIT, true);
+        }
+
+        $chunk = [];
+        foreach ($this->rawdata as $row) {
+            if ($this->hasmorecolumns) {
+                $chunk[$row->id] = $row;
+                if (count($chunk) >= self::ROWCHUNKSIZE) {
+                    $this->build_table_chunk($chunk, $columnsgroups);
+                    $chunk = [];
+                }
+            } else {
+                if ($this->buildforexternal) {
+                    $this->add_data_for_external($row);
+                } else {
+                    $this->add_data_keyed($this->format_row($row), $this->get_row_class($row));
+                }
+            }
+        }
+        $this->build_table_chunk($chunk, $columnsgroups);
+    }
+
+    /**
+     * Retrieve additional columns. Database engine may have a limit on number of joins.
+     *
+     * @param array $rows Array of rows with already retrieved data, new values will be added to this array
+     * @param array $columnsgroups array of arrays of columns. Each element has up to self::TABLEJOINLIMIT items. This
+     *     is easy to calculate but because we can call this method many times we calculate it once and pass by
+     *     reference for performance reasons
+     */
+    protected function build_table_chunk(&$rows, &$columnsgroups) {
+        global $DB;
+        if (!$rows) {
+            return;
+        }
+
+        foreach ($columnsgroups as $columnsgroup) {
+            $fields = 'c.id';
+            $from = '{feedback_completed} c';
+            $params = [];
+            foreach ($columnsgroup as $nr => $item) {
+                $fields .= ", v{$nr}.value AS val{$nr}";
+                $from .= " LEFT OUTER JOIN {feedback_value} v{$nr} " .
+                    "ON v{$nr}.completed = c.id AND v{$nr}.item = :itemid{$nr}";
+                $params["itemid{$nr}"] = $item->id;
+            }
+            list($idsql, $idparams) = $DB->get_in_or_equal(array_keys($rows), SQL_PARAMS_NAMED);
+            $sql = "SELECT $fields FROM $from WHERE c.id ".$idsql;
+            $results = $DB->get_records_sql($sql, $params + $idparams);
+            foreach ($results as $result) {
+                foreach ($result as $key => $value) {
+                    $rows[$result->id]->{$key} = $value;
+                }
+            }
+        }
+
+        foreach ($rows as $row) {
+            if ($this->buildforexternal) {
+                $this->add_data_for_external($row);
+            } else {
+                $this->add_data_keyed($this->format_row($row), $this->get_row_class($row));
+            }
+        }
+    }
+
     /**
      * Returns html code for displaying "Download" button if applicable.
      */
@@ -425,4 +581,74 @@ class mod_feedback_responses_table extends table_sql {
             return '';
         }
     }
+
+    /**
+     * Return user responses data ready for the external function.
+     *
+     * @param stdClass $row the table row containing the responses
+     * @return array returns the responses ready to be used by an external function
+     * @since Moodle 3.3
+     */
+    protected function get_responses_for_external($row) {
+        $responses = [];
+        foreach ($row as $el => $val) {
+            // Get id from column name.
+            if (preg_match('/^val(\d+)$/', $el, $matches)) {
+                $id = $matches[1];
+
+                $responses[] = [
+                    'id' => $id,
+                    'name' => $this->headers[$this->columns[$el]],
+                    'printval' => $this->other_cols($el, $row),
+                    'rawval' => $val,
+                ];
+            }
+        }
+        return $responses;
+    }
+
+    /**
+     * Add data for the external structure that will be returned.
+     *
+     * @param stdClass $row a database query record row
+     * @since Moodle 3.3
+     */
+    protected function add_data_for_external($row) {
+        $this->dataforexternal[] = [
+            'id' => $row->id,
+            'courseid' => $row->courseid,
+            'userid' => $row->userid,
+            'fullname' => fullname($row),
+            'timemodified' => $row->completed_timemodified,
+            'responses' => $this->get_responses_for_external($row),
+        ];
+    }
+
+    /**
+     * Exports the table as an external structure handling pagination.
+     *
+     * @param int $page page number (for pagination)
+     * @param int $perpage elements per page
+     * @since Moodle 3.3
+     * @return array returns the table ready to be used by an external function
+     */
+    public function export_external_structure($page = 0, $perpage = 0) {
+
+        $this->buildforexternal = true;
+        $this->add_all_values_to_output();
+        // Set-up.
+        $this->setup();
+        // Override values, if needed.
+        if ($perpage > 0) {
+            $this->pageable = true;
+            $this->currpage = $page;
+            $this->pagesize = $perpage;
+        } else {
+            $this->pagesize = $this->get_total_responses_count();
+        }
+        $this->query_db($this->pagesize, false);
+        $this->build_table();
+        $this->close_recordset();
+        return $this->dataforexternal;
+    }
 }
