@@ -27,6 +27,7 @@ $string['activityoverview'] = 'There are new forum posts';
 $string['addanewdiscussion'] = 'Add a new discussion topic';
 $string['addanewquestion'] = 'Add a new question';
 $string['addanewtopic'] = 'Add a new topic';
+$string['addtofavourites'] = 'Star this discussion';
 $string['advancedsearch'] = 'Advanced search';
 $string['allforums'] = 'All forums';
 $string['allowdiscussions'] = 'Can a {$a} post to this forum?';
@@ -39,10 +40,12 @@ $string['anyfile'] = 'Any file';
 $string['areaattachment'] = 'Attachments';
 $string['areapost'] = 'Messages';
 $string['attachment'] = 'Attachment';
+$string['attachmentname'] = 'Attachment {$a}';
 $string['attachment_help'] = 'You can optionally attach one or more files to a forum post. If you attach an image, it will be displayed after the message.';
 $string['attachmentnopost'] = 'You cannot export attachments without a post id';
 $string['attachments'] = 'Attachments';
 $string['attachmentswordcount'] = 'Attachments and word count';
+$string['availability'] = 'Availability';
 $string['blockafter'] = 'Post threshold for blocking';
 $string['blockafter_help'] = 'This setting specifies the maximum number of posts which a user can post in the given time period. Users with the capability mod/forum:postwithoutthrottling are exempt from post limits.';
 $string['blockperiod'] = 'Time period for blocking';
@@ -50,14 +53,18 @@ $string['blockperiod_help'] = 'Students can be blocked from posting more than a
 $string['blockperioddisabled'] = 'Don\'t block';
 $string['blogforum'] = 'Standard forum displayed in a blog-like format';
 $string['bynameondate'] = 'by {$a->name} - {$a->date}';
+$string['cachedef_forum_is_tracked'] = 'Forum tracking status for user';
+$string['calendardue'] = '{$a} is due';
 $string['cannotadd'] = 'Could not add the discussion for this forum';
 $string['cannotadddiscussion'] = 'Adding discussions to this forum requires group membership.';
 $string['cannotadddiscussionall'] = 'You do not have permission to add a new discussion topic for all participants.';
+$string['cannotadddiscussiongroup'] = 'You are not able to create a discussion because you are not a member of any group.';
 $string['cannotaddsubscriber'] = 'Could not add subscriber with id {$a} to this forum!';
 $string['cannotaddteacherforumto'] = 'Could not add converted teacher forum instance to section 0 in the course';
 $string['cannotcreatediscussion'] = 'Could not create new discussion';
 $string['cannotcreateinstanceforteacher'] = 'Could not create new course module instance for the teacher forum';
 $string['cannotdeletepost'] = 'You can\'t delete this post!';
+$string['cannotdeletediscussioninsinglediscussion'] = 'You cannot delete the first post in a single discussion';
 $string['cannoteditposts'] = 'You can\'t edit other people\'s posts!';
 $string['cannotfinddiscussion'] = 'Could not find the discussion in this forum';
 $string['cannotfindfirstpost'] = 'Could not find the first post in this forum';
@@ -68,26 +75,35 @@ $string['cannotmovenotvisible'] = 'Forum not visible';
 $string['cannotmovetonotexist'] = 'You can\'t move to that forum - it doesn\'t exist!';
 $string['cannotmovetonotfound'] = 'Target forum not found in this course.';
 $string['cannotmovetosingleforum'] = 'Cannot move discussion to a simple single discussion forum';
+$string['cannotpindiscussions'] = 'Sorry, you do not have permission to pin discussions.';
 $string['cannotpurgecachedrss'] = 'Could not purge the cached RSS feeds for the source and/or destination forum(s) - check your file permissionsforums';
 $string['cannotremovesubscriber'] = 'Could not remove subscriber with id {$a} from this forum!';
 $string['cannotreply'] = 'You cannot reply to this post';
 $string['cannotsplit'] = 'Discussions from this forum cannot be split';
 $string['cannotsubscribe'] = 'Sorry, but you must be a group member to subscribe.';
+$string['cannotfavourite'] = 'Sorry, but you do not have the permission to star discussions.';
 $string['cannottrack'] = 'Could not stop tracking that forum';
 $string['cannotunsubscribe'] = 'Could not unsubscribe you from that forum';
 $string['cannotupdatepost'] = 'You can not update this post';
 $string['cannotviewpostyet'] = 'You cannot read other students questions in this discussion yet because you haven\'t posted';
 $string['cannotviewusersposts'] = 'There are no posts made by this user that you are able to view.';
 $string['cleanreadtime'] = 'Mark old posts as read hour';
+$string['clicktolockdiscussion'] = 'Click to lock this discussion';
+$string['clicktounlockdiscussion'] = 'Click to unlock this discussion';
 $string['clicktounsubscribe'] = 'You are subscribed to this discussion. Click to unsubscribe.';
 $string['clicktosubscribe'] = 'You are not subscribed to this discussion. Click to subscribe.';
+$string['clicktounfavourite'] = 'You have starred this discussion. Click to unstar.';
+$string['clicktofavourite'] = 'You have not starred this discussion. Click to star.';
 $string['completiondiscussions'] = 'Student must create discussions:';
+$string['completiondiscussionsdesc'] = 'Student must create at least {$a} discussion(s)';
 $string['completiondiscussionsgroup'] = 'Require discussions';
 $string['completiondiscussionshelp'] = 'requiring discussions to complete';
 $string['completionposts'] = 'Student must post discussions or replies:';
+$string['completionpostsdesc'] = 'Student must post at least {$a} discussion(s) or reply/replies';
 $string['completionpostsgroup'] = 'Require posts';
 $string['completionpostshelp'] = 'requiring discussions or replies to complete';
 $string['completionreplies'] = 'Student must post replies:';
+$string['completionrepliesdesc'] = 'Student must post at least {$a} reply/replies';
 $string['completionrepliesgroup'] = 'Require replies';
 $string['completionreplieshelp'] = 'requiring replies to complete';
 $string['configcleanreadtime'] = 'The hour of the day to clean old posts from the \'read\' table.';
@@ -103,6 +119,7 @@ $string['configoldpostdays'] = 'Number of days old any post is considered read.'
 $string['configreplytouser'] = 'When a forum post is mailed out, should it contain the user\'s email address so that recipients can reply personally rather than via the forum? Even if set to \'Yes\' users can choose in their profile to keep their email address secret.';
 $string['configrsstypedefault'] = 'If RSS feeds are enabled, sets the default activity type.';
 $string['configrssarticlesdefault'] = 'If RSS feeds are enabled, sets the default number of articles (either discussions or posts).';
+$string['configsubscriptiontype'] = 'Default setting for subscription mode.';
 $string['configshortpost'] = 'Any post under this length (in characters not including HTML) is considered short (see below).';
 $string['configtrackingtype'] = 'Default setting for read tracking.';
 $string['configtrackreadposts'] = 'Set to \'yes\' if you want to track read/unread for each user.';
@@ -114,7 +131,12 @@ $string['confirmunsubscribe'] = 'Do you really want to unsubscribe from forum \'
 $string['couldnotadd'] = 'Could not add your post due to an unknown error';
 $string['couldnotdeletereplies'] = 'Sorry, that cannot be deleted as people have already responded to it';
 $string['couldnotupdate'] = 'Could not update your post due to an unknown error';
+$string['created'] = 'Created';
 $string['crontask'] = 'Forum mailings and maintenance jobs';
+$string['cutoffdate'] = 'Cut-off date';
+$string['cutoffdate_help'] = 'If set, the forum will not accept posts after this date.';
+$string['cutoffdatereached'] = 'The cut-off date for posting to this forum is reached so you can no longer post to it.';
+$string['cutoffdatevalidation'] = 'The cut-off date cannot be earlier than the due date.';
 $string['delete'] = 'Delete';
 $string['deleteddiscussion'] = 'The discussion topic has been deleted';
 $string['deletedpost'] = 'The post has been deleted';
@@ -133,6 +155,12 @@ $string['disallowsubscription'] = 'Subscription';
 $string['disallowsubscription_help'] = 'This forum has been configured so that you cannot subscribe to discussions.';
 $string['disallowsubscribeteacher'] = 'Subscriptions not allowed (except for teachers)';
 $string['discussion'] = 'Discussion';
+$string['discussionlistsortbycreatedasc'] = 'Sort the discussions list by the creation date in ascending order';
+$string['discussionlistsortbycreateddesc'] = 'Sort the discussions list by the creation date in descending order';
+$string['discussionlistsortbylastpostdesc'] = 'Sort the discussions list by the creation date of the last post in descending order';
+$string['discussionlistsortbylastpostasc'] = 'Sort the discussions list by the creation date of the last post in ascending order';
+$string['discussionlistsortbyrepliesasc'] = 'Sort the discussions list by the number of replies in ascending order';
+$string['discussionlistsortbyrepliesdesc'] = 'Sort the discussions list by the number of replies in descending order';
 $string['discussionlocked'] = 'This discussion has been locked so you can no longer reply to it.';
 $string['discussionlockingheader'] = 'Discussion locking';
 $string['discussionlockingdisabled'] = 'Do not lock discussions';
@@ -144,6 +172,7 @@ $string['discussionnowsubscribed'] = '{$a->name} will be notified of new posts i
 $string['discussionpin'] = 'Pin';
 $string['discussionpinned'] = 'Pinned';
 $string['discussionpinned_help'] = 'Pinned discussions will appear at the top of a forum.';
+$string['discussionsplit'] = 'Discussion has been split';
 $string['discussionsubscribestop'] = 'I don\'t want to be notified of new posts in this discussion';
 $string['discussionsubscribestart'] = 'Send me notifications of new posts in this discussion';
 $string['discussionsubscription'] = 'Discussion subscription';
@@ -162,6 +191,9 @@ $string['displaystart'] = 'Display start';
 $string['displaystart_help'] = 'This setting specifies whether a forum post should be displayed from a certain date. Note that administrators can always view forum posts.';
 $string['displaywordcount'] = 'Display word count';
 $string['displaywordcount_help'] = 'This setting specifies whether the word count of each post should be displayed or not.';
+$string['duedate'] = 'Due date';
+$string['duedate_help'] = 'This is when posting in the forum is due. Although this date is displayed in the calendar as the due date for the forum, posting will still be allowed after this date. Set a forum cut-off date to prevent posting to the forum after a certain date.';
+$string['duedatetodisplayincalendar'] = 'Due date to display in calendar';
 $string['eachuserforum'] = 'Each person posts one discussion';
 $string['edit'] = 'Edit';
 $string['editedby'] = 'Edited by {$a->name} - original submission {$a->date}';
@@ -203,6 +235,7 @@ $string['erroremptymessage'] = 'Post message cannot be empty';
 $string['erroremptysubject'] = 'Post subject cannot be empty.';
 $string['errorenrolmentrequired'] = 'You must be enrolled in this course to access this content';
 $string['errorwhiledelete'] = 'An error occurred while deleting record.';
+$string['errorcannotlock'] = 'You do not have the permission to lock discussions.';
 $string['eventassessableuploaded'] = 'Some content has been posted.';
 $string['everyonecanchoose'] = 'Everyone can choose to be subscribed';
 $string['everyonecannowchoose'] = 'Everyone can now choose to be subscribed';
@@ -210,6 +243,10 @@ $string['everyoneisnowsubscribed'] = 'Everyone is now subscribed to this forum';
 $string['everyoneissubscribed'] = 'Everyone is subscribed to this forum';
 $string['existingsubscribers'] = 'Existing subscribers';
 $string['exportdiscussion'] = 'Export whole discussion to portfolio';
+$string['exportattachmentname'] = 'Export attachment {$a} to portfolio';
+$string['firstpost'] = 'First post';
+$string['favourites'] = 'Starred';
+$string['favouriteupdated'] = 'Your star option has been updated.';
 $string['forcedreadtracking'] = 'Allow forced read tracking';
 $string['forcedreadtracking_desc'] = 'Allows forums to be set to forced read tracking. Will result in decreased performance for some users, particularly on courses with many forums and posts. When off, any forums previously set to Forced are treated as optional.';
 $string['forcesubscribed_help'] = 'This forum has been configured so that you cannot unsubscribe from discussions.';
@@ -219,11 +256,13 @@ $string['forum:addinstance'] = 'Add a new forum';
 $string['forum:addnews'] = 'Add announcements';
 $string['forum:addquestion'] = 'Add question';
 $string['forum:allowforcesubscribe'] = 'Allow force subscribe';
+$string['forum:canoverridecutoff'] = 'Post to forums after their cut-off date';
 $string['forum:canoverridediscussionlock'] = 'Reply to locked discussions';
+$string['forum:cantogglefavourite'] = 'Star discussions';
 $string['forumauthorhidden'] = 'Author (hidden)';
 $string['forumblockingalmosttoomanyposts'] = 'You are approaching the posting threshold. You have posted {$a->numposts} times in the last {$a->blockperiod} and the limit is {$a->blockafter} posts.';
 $string['forumbodyhidden'] = 'This post cannot be viewed by you, probably because you have not posted in the discussion, the maximum editing time hasn\'t passed yet, the discussion has not started or the discussion has expired.';
-$string['forum:canposttomygroups'] = 'Can post to all groups you have access to';
+$string['forum:canposttomygroups'] = 'Post to all groups you have access to';
 $string['forum:createattachment'] = 'Create attachments';
 $string['forum:deleteanypost'] = 'Delete any posts (anytime)';
 $string['forum:deleteownpost'] = 'Delete own posts (within deadline)';
@@ -232,7 +271,7 @@ $string['forum:exportdiscussion'] = 'Export whole discussion';
 $string['forum:exportownpost'] = 'Export own post';
 $string['forum:exportpost'] = 'Export post';
 $string['forumintro'] = 'Description';
-$string['forum:managesubscriptions'] = 'Manage subscriptions';
+$string['forum:managesubscriptions'] = 'Manage subscribers';
 $string['forum:movediscussions'] = 'Move discussions';
 $string['forum:pindiscussions'] = 'Pin discussions';
 $string['forum:postwithoutthrottling'] = 'Exempt from post threshold';
@@ -241,6 +280,8 @@ $string['forumposts'] = 'Forum posts';
 $string['forum:rate'] = 'Rate posts';
 $string['forum:replynews'] = 'Reply to announcements';
 $string['forum:replypost'] = 'Reply to posts';
+$string['forum:postprivatereply'] = 'Reply privately to posts';
+$string['forum:readprivatereplies'] = 'View private replies';
 $string['forums'] = 'Forums';
 $string['forum:splitdiscussions'] = 'Split discussions';
 $string['forum:startdiscussion'] = 'Start new discussions';
@@ -265,7 +306,12 @@ $string['forum:viewsubscribers'] = 'View subscribers';
 $string['generalforum'] = 'Standard forum for general use';
 $string['generalforums'] = 'General forums';
 $string['hiddenforumpost'] = 'Hidden forum post';
+$string['indicator:cognitivedepth'] = 'Forum cognitive';
+$string['indicator:cognitivedepth_help'] = 'This indicator is based on the cognitive depth reached by the student in a Forum activity.';
+$string['indicator:socialbreadth'] = 'Forum social';
+$string['indicator:socialbreadth_help'] = 'This indicator is based on the social breadth reached by the student in a Forum activity.';
 $string['inforum'] = 'in {$a}';
+$string['inreplyto'] = 'In reply to {$a}';
 $string['introblog'] = 'The posts in this forum were copied here automatically from blogs of users in this course because those blog entries are no longer available';
 $string['intronews'] = 'General news and announcements';
 $string['introsocial'] = 'An open forum for chatting about anything you want to';
@@ -277,6 +323,7 @@ $string['invalidforcesubscribe'] = 'Invalid force subscription mode';
 $string['invalidforumid'] = 'Forum ID was incorrect';
 $string['invalidparentpostid'] = 'Parent post ID was incorrect';
 $string['invalidpostid'] = 'Invalid post ID - {$a}';
+$string['inpagereplysubject'] = 'Re: {$a}';
 $string['lastpost'] = 'Last post';
 $string['learningforums'] = 'Learning forums';
 $string['lockdiscussionafter'] = 'Lock discussions after period of inactivity';
@@ -284,8 +331,13 @@ $string['lockdiscussionafter_help'] = 'Discussions may be automatically locked a
 
 Users with the capability to reply to locked discussions can unlock a discussion by replying to it.';
 $string['longpost'] = 'Long post';
+$string['locked'] = 'Locked';
+$string['lockdiscussion'] = 'Lock this discussion';
+$string['lockupdated'] = 'The lock option has been updated.';
 $string['mailnow'] = 'Send forum post notifications with no editing-time delay';
 $string['manydiscussions'] = 'Discussions per page';
+$string['managesubscriptionsoff'] = 'Finish managing subscriptions';
+$string['managesubscriptionson'] = 'Manage subscribers';
 $string['markalldread'] = 'Mark all posts in this discussion read.';
 $string['markallread'] = 'Mark all posts in this forum read.';
 $string['markasreadonnotification'] = 'When sending forum post notifications';
@@ -372,16 +424,18 @@ $string['notexists'] = 'Discussion no longer exists';
 $string['nothingnew'] = 'Nothing new for {$a}';
 $string['notingroup'] = 'Sorry, but you need to be part of a group to see this forum.';
 $string['notinstalled'] = 'The forum module is not installed';
+$string['notlocked'] = 'Lock';
 $string['notpartofdiscussion'] = 'This post is not part of a discussion!';
 $string['notrackforum'] = 'Don\'t track unread posts';
 $string['noviewdiscussionspermission'] = 'You do not have the permission to view discussions in this forum';
-$string['nowallsubscribed'] = 'All forums in {$a} are subscribed.';
-$string['nowallunsubscribed'] = 'All forums in {$a} are not subscribed.';
+$string['nowallsubscribed'] = 'You are now subscribed to all forums in {$a}.';
+$string['nowallunsubscribed'] = 'You are now unsubscribed from all forums in {$a}.';
 $string['nownotsubscribed'] = '{$a->name} will NOT be notified of new posts in \'{$a->forum}\'';
 $string['nownottracking'] = '{$a->name} is no longer tracking \'{$a->forum}\'.';
 $string['nowsubscribed'] = '{$a->name} will be notified of new posts in \'{$a->forum}\'';
 $string['nowtracking'] = '{$a->name} is now tracking \'{$a->forum}\'.';
 $string['numposts'] = '{$a} posts';
+$string['numberofreplies'] = 'Number of replies: {$a}';
 $string['olderdiscussions'] = 'Older discussions';
 $string['oldertopics'] = 'Older topics';
 $string['oldpostdays'] = 'Read after days';
@@ -393,6 +447,11 @@ $string['page-mod-forum-discuss'] = 'Forum module discussion thread page';
 $string['parent'] = 'Show parent';
 $string['parentofthispost'] = 'Parent of this post';
 $string['permalink'] = 'Permalink';
+$string['permanentlinktopost'] = 'Permanent link to this post';
+$string['permanentlinktoparentpost'] = 'Permanent link to the parent of this post';
+$string['postisprivatereply'] = 'This is a private reply. It is not visible to other participants.';
+$string['pindiscussion'] = 'Pin this discussion';
+$string['pinupdated'] = 'The pin option has been updated.';
 $string['posttomygroups'] = 'Post a copy to all groups';
 $string['posttomygroups_help'] = 'Posts a copy of this message to all groups you have access to. Participants in groups you do not have access to will not see this post';
 $string['prevdiscussiona'] = 'Previous discussion: {$a}';
@@ -419,6 +478,65 @@ $string['postsmadebyuserincourse'] = 'Posts made by {$a->fullname} in {$a->cours
 $string['posttoforum'] = 'Post to forum';
 $string['postupdated'] = 'Your post was updated';
 $string['potentialsubscribers'] = 'Potential subscribers';
+$string['privacy:digesttypenone'] = 'We do not hold any data relating to a preferred forum digest type for this forum.';
+$string['privacy:digesttypepreference'] = 'You have chosen to receive the following forum digest type: "{$a->type}".';
+$string['privacy:discussionsubscriptionpreference'] = 'You have chosen the following discussion subscription preference for this forum: "{$a->preference}"';
+$string['privacy:metadata:core_tag'] = 'The forum makes use of the tag subsystem to support tagging of posts.';
+$string['privacy:metadata:core_rating'] = 'The forum makes use of the rating subsystem to support the rating of posts.';
+$string['privacy:metadata:forum_digests'] = 'Information about the digest preferences for each forum.';
+$string['privacy:metadata:forum_digests:forum'] = 'The forum subscribed to.';
+$string['privacy:metadata:forum_digests:maildigest'] = 'The digest preference.';
+$string['privacy:metadata:forum_digests:userid'] = 'The ID of the user with the digest preference.';
+$string['privacy:metadata:forum_discussion_subs'] = 'Information about the subscriptions to individual forum discussions';
+$string['privacy:metadata:forum_discussion_subs:discussionid'] = 'The ID of the discussion that was subscribed to.';
+$string['privacy:metadata:forum_discussion_subs:preference'] = 'The start time of the subscription.';
+$string['privacy:metadata:forum_discussion_subs:userid'] = 'The ID of the user with the discussion subscription.';
+$string['privacy:metadata:forum_discussions'] = 'Information about the individual forum discussions that a user has created';
+$string['privacy:metadata:forum_discussions:assessed'] = 'TODOD - what does this field store';
+$string['privacy:metadata:forum_discussions:name'] = 'The name of the discussion, as chosen by the author.';
+$string['privacy:metadata:forum_discussions:timemodified'] = 'The time that the discussion was last modified.';
+$string['privacy:metadata:forum_discussions:userid'] = 'The ID of the user who created the discussion';
+$string['privacy:metadata:forum_discussions:usermodified'] = 'The ID of the user who last modified the discussion in some way.';
+$string['privacy:metadata:forum_posts'] = 'Information about the digest preferences for each forum.';
+$string['privacy:metadata:forum_posts:created'] = 'The time that the post was created.';
+$string['privacy:metadata:forum_posts:discussion'] = 'The discussion that the post is in.';
+$string['privacy:metadata:forum_posts:message'] = 'The message of the forum post.';
+$string['privacy:metadata:forum_posts:modified'] = 'The time that the post was last modified.';
+$string['privacy:metadata:forum_posts:parent'] = 'The parent post that was replied to.';
+$string['privacy:metadata:forum_posts:subject'] = 'The subject of the forum post.';
+$string['privacy:metadata:forum_posts:totalscore'] = 'The message of the forum post.';
+$string['privacy:metadata:forum_posts:userid'] = 'The ID of the user who authored the forum post.';
+$string['privacy:metadata:forum_posts:privatereplyto'] = 'The ID of the user this reply was sent to.';
+$string['privacy:metadata:forum_queue'] = 'Temporary log of posts that will be mailed in digest form';
+$string['privacy:metadata:forum_queue:discussionid'] = 'Forum discussion ID';
+$string['privacy:metadata:forum_queue:postid'] = 'Forum post ID';
+$string['privacy:metadata:forum_queue:timemodified'] = 'The modified time of the original post';
+$string['privacy:metadata:forum_queue:userid'] = 'User who needs to be notified of the post';
+$string['privacy:metadata:forum_read'] = 'Information about which posts have been read by the user.';
+$string['privacy:metadata:forum_read:discussionid'] = 'The discussion that the post is in.';
+$string['privacy:metadata:forum_read:firstread'] = 'The first time that the post was read.';
+$string['privacy:metadata:forum_read:lastread'] = 'The most recent time that the post was read.';
+$string['privacy:metadata:forum_read:postid'] = 'The post that was read.';
+$string['privacy:metadata:forum_read:userid'] = 'The ID of the user that this record relates to.';
+$string['privacy:metadata:forum_subscriptions'] = 'Information about which forums the user has subscribed to.';
+$string['privacy:metadata:forum_subscriptions:forum'] = 'The forum that was subscribed to.';
+$string['privacy:metadata:forum_subscriptions:userid'] = 'The ID of the user that this forum subscription relates to.';
+$string['privacy:metadata:forum_track_prefs'] = 'Information about which forums the user has chosen to track post reads for.';
+$string['privacy:metadata:forum_track_prefs:forumid'] = 'The forum that has read tracking enabled.';
+$string['privacy:metadata:forum_track_prefs:userid'] = 'The ID of the user that this forum tracking preference relates to.';
+$string['privacy:metadata:preference:autosubscribe'] = 'Whether to subscribe to discussions when replying to posts within them.';
+$string['privacy:metadata:preference:forum_discussionlistsortorder'] = 'The preferred sorting order of the discussions list';
+$string['privacy:metadata:preference:maildigest'] = 'The site-wide mail digest preference';
+$string['privacy:metadata:preference:markasreadonnotification'] = 'Whether to mark forum posts as read when receiving them as messages.';
+$string['privacy:metadata:preference:trackforums'] = 'Whether to enable read tracking.';
+$string['privacy:postwasread'] = 'This post was first read on {$a->firstread} and most recently read on {$a->lastread}';
+$string['privacy:readtrackingdisabled'] = 'You have chosen to not track posts you have read within this forum.';
+$string['privacy:request:delete:discussion:name'] = 'Delete at the request of the author';
+$string['privacy:request:delete:post:message'] = 'The content of this post has been deleted at the request of its author.';
+$string['privacy:request:delete:post:subject'] = 'Delete at the request of the author';
+$string['privacy:subscribedtoforum'] = 'You are subscribed to this forum.';
+$string['privatereply'] = 'Reply privately';
+$string['privatereply_help'] = 'A private reply can only be viewed by the author of the post being replied to, and any users with the capability to view private replies.';
 $string['processingdigest'] = 'Processing email digest for user {$a}';
 $string['processingpost'] = 'Processing post {$a}';
 $string['prune'] = 'Split';
@@ -428,10 +546,13 @@ $string['qandaforum'] = 'Q and A forum';
 $string['qandanotify'] = 'This is a question and answer forum. In order to see other responses to these questions, you must first post your answer';
 $string['re'] = 'Re:';
 $string['readtherest'] = 'Read the rest of this topic';
+$string['removeallforumtags'] = 'Remove all forum tags';
+$string['removefromfavourites'] = 'Unstar this discussion';
 $string['replies'] = 'Replies';
 $string['repliesmany'] = '{$a} replies so far';
 $string['repliesone'] = '{$a} reply so far';
 $string['reply'] = 'Reply';
+$string['replyplaceholder'] = 'Write your reply...';
 $string['replyforum'] = 'Reply to forum';
 $string['replytopostbyemail'] = 'You can reply to this via email.';
 $string['replytouser'] = 'Use email address in reply';
@@ -462,12 +583,15 @@ $string['searcholderposts'] = 'Search older posts...';
 $string['searchphrase'] = 'This exact phrase must appear in the post';
 $string['searchresults'] = 'Search results';
 $string['searchsubject'] = 'These words should be in the subject';
+$string['searchtags'] = 'Is tagged with';
 $string['searchuser'] = 'This name should match the author';
 $string['searchuserid'] = 'The Moodle ID of the author';
 $string['searchwhichforums'] = 'Choose which forums to search';
 $string['searchwords'] = 'These words can appear anywhere in the post';
 $string['seeallposts'] = 'See all posts made by this user';
+$string['settings'] = 'Settings';
 $string['shortpost'] = 'Short post';
+$string['showingcountoftotaldiscussions'] = 'Showing {$a->count} of {$a->total} discussions';
 $string['showsubscribers'] = 'Show/edit current subscribers';
 $string['singleforum'] = 'A single simple discussion';
 $string['smallmessage'] = '{$a->user} posted in {$a->forumname}';
@@ -493,15 +617,17 @@ $string['subscriptionmode_help'] = 'When a participant is subscribed to a forum
 * Optional subscription - Participants can choose whether to be subscribed
 * Forced subscription - Everyone is subscribed and cannot unsubscribe
 * Auto subscription - Everyone is subscribed initially but can choose to unsubscribe at any time
-* Subscription disabled - Subscriptions are not allowed
-
-Note: Any subscription mode changes will only affect users who enrol in the course in the future, and not existing users.';
+* Subscription disabled - Subscriptions are not allowed';
 $string['subscriptionoptional'] = 'Optional subscription';
 $string['subscriptionforced'] = 'Forced subscription';
 $string['subscriptionauto'] = 'Auto subscription';
 $string['subscriptiondisabled'] = 'Subscription disabled';
 $string['subscriptions'] = 'Subscriptions';
+$string['tagarea_forum_posts'] = 'Forum posts';
+$string['tagsdeleted'] = 'Forum tags have been deleted';
 $string['thisforumisthrottled'] = 'This forum has a limit to the number of forum postings you can make in a given time period - this is currently set at {$a->blockafter} posting(s) in {$a->blockperiod}';
+$string['thisforumisdue'] = 'The due date for posting to this forum was {$a}.';
+$string['thisforumhasduedate'] = 'The due date for posting to this forum is {$a}.';
 $string['timedhidden'] = 'Timed status: Hidden from students';
 $string['timedposts'] = 'Timed posts';
 $string['timedvisible'] = 'Timed status: Visible to all users';
@@ -518,6 +644,7 @@ $string['trackingtype_help'] = 'Read tracking enables participants to easily che
 If set to optional, participants can choose whether to turn tracking on or off via a link in the administration block. (Users must also enable forum tracking in their forum preferences.)
 
 If \'Allow forced read tracking\' is enabled in the site administration, then a further option is available - forced. This means that tracking is always on, regardless of users\' forum preferences.';
+$string['unlockdiscussion'] = 'Unlock this discussion';
 $string['unread'] = 'Unread';
 $string['unreadposts'] = 'Unread posts';
 $string['unreadpostsnumber'] = '{$a} unread posts';
@@ -535,6 +662,7 @@ $string['unsubscribeallempty'] = 'You are not subscribed to any forums. To disab
 $string['unsubscribed'] = 'Unsubscribed';
 $string['unsubscribeshort'] = 'Unsubscribe';
 $string['usermarksread'] = 'Manual message read marking';
+$string['unpindiscussion'] = 'Unpin this discussion';
 $string['viewalldiscussions'] = 'View all discussions';
 $string['viewthediscussion'] = 'View the discussion';
 $string['warnafter'] = 'Post threshold for warning';
@@ -543,16 +671,5 @@ $string['warnformorepost'] = 'Warning! There is more than one discussion in this
 $string['yournewquestion'] = 'Your new question';
 $string['yournewtopic'] = 'Your new discussion topic';
 $string['yourreply'] = 'Your reply';
-
-// Deprecated since Moodle 3.0.
-$string['subscribersto'] = 'Subscribers to "{$a->name}"';
-
-// Deprecated since Moodle 3.1.
-$string['postmailinfo'] = 'This is a copy of a message posted on the {$a} website.
-
-To reply click on this link:';
-$string['emaildigestupdated'] = 'The e-mail digest option was changed to \'{$a->maildigesttitle}\' for the forum \'{$a->forum}\'. {$a->maildigestdescription}';
-$string['emaildigestupdated_default'] = 'Your default profile setting of \'{$a->maildigesttitle}\' was used for the forum \'{$a->forum}\'. {$a->maildigestdescription}.';
-$string['emaildigest_0'] = 'You will receive one e-mail per forum post.';
-$string['emaildigest_1'] = 'You will receive one digest e-mail per day containing the complete contents of each forum post.';
-$string['emaildigest_2'] = 'You will receive one digest e-mail per day containing the subject of each forum post.';
+$string['forumsubjectdeleted'] = 'This forum post has been removed';
+$string['forumbodydeleted'] = 'The content of this forum post has been removed and can no longer be accessed.';
