@@ -25,14 +25,20 @@
 defined('MOODLE_INTERNAL') || die();
 
 global $CFG;
-require_once($CFG->dirroot . '/mod/forum/lib.php');
+require_once(__DIR__ . '/generator_trait.php');
+require_once("{$CFG->dirroot}/mod/forum/lib.php");
 
 class mod_forum_subscriptions_testcase extends advanced_testcase {
+    // Include the mod_forum test helpers.
+    // This includes functions to create forums, users, discussions, and posts.
+    use mod_forum_tests_generator_trait;
 
     /**
      * Test setUp.
      */
     public function setUp() {
+        global $DB;
+
         // We must clear the subscription caches. This has to be done both before each test, and after in case of other
         // tests using these functions.
         \mod_forum\subscriptions::reset_forum_cache();
@@ -49,52 +55,6 @@ class mod_forum_subscriptions_testcase extends advanced_testcase {
         \mod_forum\subscriptions::reset_discussion_cache();
     }
 
-    /**
-     * Helper to create the required number of users in the specified
-     * course.
-     * Users are enrolled as students.
-     *
-     * @param stdClass $course The course object
-     * @param integer $count The number of users to create
-     * @return array The users created
-     */
-    protected function helper_create_users($course, $count) {
-        $users = array();
-
-        for ($i = 0; $i < $count; $i++) {
-            $user = $this->getDataGenerator()->create_user();
-            $this->getDataGenerator()->enrol_user($user->id, $course->id);
-            $users[] = $user;
-        }
-
-        return $users;
-    }
-
-    /**
-     * Create a new discussion and post within the specified forum, as the
-     * specified author.
-     *
-     * @param stdClass $forum The forum to post in
-     * @param stdClass $author The author to post as
-     * @param array An array containing the discussion object, and the post object
-     */
-    protected function helper_post_to_forum($forum, $author) {
-        global $DB;
-        $generator = $this->getDataGenerator()->get_plugin_generator('mod_forum');
-
-        // Create a discussion in the forum, and then add a post to that discussion.
-        $record = new stdClass();
-        $record->course = $forum->course;
-        $record->userid = $author->id;
-        $record->forum = $forum->id;
-        $discussion = $generator->create_discussion($record);
-
-        // Retrieve the post which was created by create_discussion.
-        $post = $DB->get_record('forum_posts', array('discussion' => $discussion->id));
-
-        return array($discussion, $post);
-    }
-
     public function test_subscription_modes() {
         global $DB;
 
@@ -106,6 +66,12 @@ class mod_forum_subscriptions_testcase extends advanced_testcase {
         $options = array('course' => $course->id);
         $forum = $this->getDataGenerator()->create_module('forum', $options);
 
+        // Create a user enrolled in the course as a student.
+        list($user) = $this->helper_create_users($course, 1);
+
+        // Must be logged in as the current user.
+        $this->setUser($user);
+
         \mod_forum\subscriptions::set_subscription_mode($forum->id, FORUM_FORCESUBSCRIBE);
         $forum = $DB->get_record('forum', array('id' => $forum->id));
         $this->assertEquals(FORUM_FORCESUBSCRIBE, \mod_forum\subscriptions::get_subscription_mode($forum));
@@ -189,7 +155,6 @@ class mod_forum_subscriptions_testcase extends advanced_testcase {
         $roleids = $DB->get_records_menu('role', null, '', 'shortname, id');
         $context = \context_course::instance($course->id);
         assign_capability('moodle/course:viewhiddenactivities', CAP_ALLOW, $roleids['student'], $context);
-        $context->mark_dirty();
 
         // All of the unsubscribable forums should now be listed.
         $result = \mod_forum\subscriptions::get_unsubscribable_forums();
@@ -220,11 +185,6 @@ class mod_forum_subscriptions_testcase extends advanced_testcase {
         // Check that the user is currently not subscribed to the forum.
         $this->assertFalse(\mod_forum\subscriptions::is_subscribed($author->id, $forum));
 
-        // Check the deprecated function too.
-        $this->assertFalse(forum_is_subscribed($author->id, $forum));
-        $this->assertEquals(1, count($this->getDebuggingMessages()));
-        $this->resetDebugging();
-
         // Check that the user is unsubscribed from the discussion too.
         $this->assertFalse(\mod_forum\subscriptions::is_subscribed($author->id, $forum, $discussion->id));
 
@@ -262,36 +222,6 @@ class mod_forum_subscriptions_testcase extends advanced_testcase {
             'discussion'    => $discussion->id,
         )));
 
-        // The same thing should happen calling the deprecated versions of
-        // these functions.
-        // Subscribing to the forum should create a record in the subscriptions table, but not the forum discussion
-        // subscriptions table.
-        forum_subscribe($author->id, $forum->id);
-        $this->assertEquals(1, count($this->getDebuggingMessages()));
-        $this->resetDebugging();
-        $this->assertEquals(1, $DB->count_records('forum_subscriptions', array(
-            'userid'        => $author->id,
-            'forum'         => $forum->id,
-        )));
-        $this->assertEquals(0, $DB->count_records('forum_discussion_subs', array(
-            'userid'        => $author->id,
-            'discussion'    => $discussion->id,
-        )));
-
-        // Unsubscribing should remove the record from the forum subscriptions table, and not modify the forum
-        // discussion subscriptions table.
-        forum_unsubscribe($author->id, $forum->id);
-        $this->assertEquals(1, count($this->getDebuggingMessages()));
-        $this->resetDebugging();
-        $this->assertEquals(0, $DB->count_records('forum_subscriptions', array(
-            'userid'        => $author->id,
-            'forum'         => $forum->id,
-        )));
-        $this->assertEquals(0, $DB->count_records('forum_discussion_subs', array(
-            'userid'        => $author->id,
-            'discussion'    => $discussion->id,
-        )));
-
         // Enroling the user in the discussion should add one record to the forum discussion table without modifying the
         // form subscriptions.
         \mod_forum\subscriptions::subscribe_user_to_discussion($author->id, $discussion);
@@ -439,11 +369,6 @@ class mod_forum_subscriptions_testcase extends advanced_testcase {
         // Check that the user is currently not subscribed to the forum.
         $this->assertFalse(\mod_forum\subscriptions::is_subscribed($author->id, $forum));
 
-        // Check the deprecated function too.
-        $this->assertFalse(forum_is_subscribed($author->id, $forum));
-        $this->assertEquals(1, count($this->getDebuggingMessages()));
-        $this->resetDebugging();
-
         // Post a discussion to the forum.
         list($discussion, $post) = $this->helper_post_to_forum($forum, $author);
 
@@ -476,11 +401,6 @@ class mod_forum_subscriptions_testcase extends advanced_testcase {
         // Check that the user is currently subscribed to the forum.
         $this->assertTrue(\mod_forum\subscriptions::is_subscribed($author->id, $forum));
 
-        // Check the deprecated function too.
-        $this->assertTrue(forum_is_subscribed($author->id, $forum));
-        $this->assertEquals(1, count($this->getDebuggingMessages()));
-        $this->resetDebugging();
-
         // Post a discussion to the forum.
         list($discussion, $post) = $this->helper_post_to_forum($forum, $author);
 
@@ -506,11 +426,6 @@ class mod_forum_subscriptions_testcase extends advanced_testcase {
         // Check that the user is currently not subscribed to the forum.
         $this->assertFalse(\mod_forum\subscriptions::is_subscribed($author->id, $forum));
 
-        // Check the deprecated function too.
-        $this->assertFalse(forum_is_subscribed($author->id, $forum));
-        $this->assertEquals(1, count($this->getDebuggingMessages()));
-        $this->resetDebugging();
-
         // Post a discussion to the forum.
         list($discussion, $post) = $this->helper_post_to_forum($forum, $author);
 
@@ -523,11 +438,6 @@ class mod_forum_subscriptions_testcase extends advanced_testcase {
         // Check that the user is still unsubscribed from the forum.
         $this->assertFalse(\mod_forum\subscriptions::is_subscribed($author->id, $forum));
 
-        // Check the deprecated function too.
-        $this->assertFalse(forum_is_subscribed($author->id, $forum));
-        $this->assertEquals(1, count($this->getDebuggingMessages()));
-        $this->resetDebugging();
-
         // But subscribed to the discussion.
         $this->assertTrue(\mod_forum\subscriptions::is_subscribed($author->id, $forum, $discussion->id));
     }
@@ -553,11 +463,6 @@ class mod_forum_subscriptions_testcase extends advanced_testcase {
         // Check that the user is currently subscribed to the forum.
         $this->assertTrue(\mod_forum\subscriptions::is_subscribed($author->id, $forum));
 
-        // Check the deprecated function too.
-        $this->assertTrue(forum_is_subscribed($author->id, $forum));
-        $this->assertEquals(1, count($this->getDebuggingMessages()));
-        $this->resetDebugging();
-
         // Post a discussion to the forum.
         list($discussion, $post) = $this->helper_post_to_forum($forum, $author);
 
@@ -567,11 +472,6 @@ class mod_forum_subscriptions_testcase extends advanced_testcase {
         // Check that the user is still subscribed to the forum.
         $this->assertTrue(\mod_forum\subscriptions::is_subscribed($author->id, $forum));
 
-        // Check the deprecated function too.
-        $this->assertTrue(forum_is_subscribed($author->id, $forum));
-        $this->assertEquals(1, count($this->getDebuggingMessages()));
-        $this->resetDebugging();
-
         // But unsubscribed from the discussion.
         $this->assertFalse(\mod_forum\subscriptions::is_subscribed($author->id, $forum, $discussion->id));
     }
@@ -599,11 +499,6 @@ class mod_forum_subscriptions_testcase extends advanced_testcase {
         // Check that the user is currently subscribed to the forum.
         $this->assertTrue(\mod_forum\subscriptions::is_subscribed($author->id, $forum));
 
-        // Check the deprecated function too.
-        $this->assertTrue(forum_is_subscribed($author->id, $forum));
-        $this->assertEquals(1, count($this->getDebuggingMessages()));
-        $this->resetDebugging();
-
         // Post a discussion to the forum.
         list($discussion, $post) = $this->helper_post_to_forum($forum, $author);
 
@@ -629,11 +524,6 @@ class mod_forum_subscriptions_testcase extends advanced_testcase {
         // Check that the user is still subscribed to the forum.
         $this->assertTrue(\mod_forum\subscriptions::is_subscribed($author->id, $forum));
 
-        // Check the deprecated function too.
-        $this->assertTrue(forum_is_subscribed($author->id, $forum));
-        $this->assertEquals(1, count($this->getDebuggingMessages()));
-        $this->resetDebugging();
-
         // An attempt to unsubscribe again should result in a falsey return to indicate that no change was made.
         $this->assertFalse(\mod_forum\subscriptions::unsubscribe_user_from_discussion($author->id, $discussion));
 
@@ -668,16 +558,6 @@ class mod_forum_subscriptions_testcase extends advanced_testcase {
         // Check that the user is still subscribed to the forum.
         $this->assertTrue(\mod_forum\subscriptions::is_subscribed($author->id, $forum));
 
-        // Check the deprecated function too.
-        $this->assertTrue(forum_is_subscribed($author->id, $forum));
-        $this->assertEquals(1, count($this->getDebuggingMessages()));
-        $this->resetDebugging();
-
-        // Check the deprecated function too.
-        $this->assertTrue(forum_is_subscribed($author->id, $forum));
-        $this->assertEquals(1, count($this->getDebuggingMessages()));
-        $this->resetDebugging();
-
         // And is subscribed to the discussion again.
         $this->assertTrue(\mod_forum\subscriptions::is_subscribed($author->id, $forum, $discussion->id));
 
@@ -699,11 +579,6 @@ class mod_forum_subscriptions_testcase extends advanced_testcase {
         // Check that the user is still subscribed to the forum.
         $this->assertTrue(\mod_forum\subscriptions::is_subscribed($author->id, $forum));
 
-        // Check the deprecated function too.
-        $this->assertTrue(forum_is_subscribed($author->id, $forum));
-        $this->assertEquals(1, count($this->getDebuggingMessages()));
-        $this->resetDebugging();
-
         // But unsubscribed from the discussion.
         $this->assertFalse(\mod_forum\subscriptions::is_subscribed($author->id, $forum, $discussion->id));
 
@@ -747,11 +622,6 @@ class mod_forum_subscriptions_testcase extends advanced_testcase {
         // Check that the user is still subscribed to the forum.
         $this->assertTrue(\mod_forum\subscriptions::is_subscribed($author->id, $forum));
 
-        // Check the deprecated function too.
-        $this->assertTrue(forum_is_subscribed($author->id, $forum));
-        $this->assertEquals(1, count($this->getDebuggingMessages()));
-        $this->resetDebugging();
-
         // But unsubscribed from the discussion.
         $this->assertFalse(\mod_forum\subscriptions::is_subscribed($author->id, $forum, $discussion->id));
 
@@ -806,11 +676,6 @@ class mod_forum_subscriptions_testcase extends advanced_testcase {
         // Check that the user is currently unsubscribed to the forum.
         $this->assertFalse(\mod_forum\subscriptions::is_subscribed($author->id, $forum));
 
-        // Check the deprecated function too.
-        $this->assertFalse(forum_is_subscribed($author->id, $forum));
-        $this->assertEquals(1, count($this->getDebuggingMessages()));
-        $this->resetDebugging();
-
         // Post a discussion to the forum.
         list($discussion, $post) = $this->helper_post_to_forum($forum, $author);
 
@@ -826,11 +691,6 @@ class mod_forum_subscriptions_testcase extends advanced_testcase {
         // Check that the user is still unsubscribed from the forum.
         $this->assertFalse(\mod_forum\subscriptions::is_subscribed($author->id, $forum));
 
-        // Check the deprecated function too.
-        $this->assertFalse(forum_is_subscribed($author->id, $forum));
-        $this->assertEquals(1, count($this->getDebuggingMessages()));
-        $this->resetDebugging();
-
         // But subscribed to the discussion.
         $this->assertTrue(\mod_forum\subscriptions::is_subscribed($author->id, $forum, $discussion->id));
 
@@ -846,11 +706,6 @@ class mod_forum_subscriptions_testcase extends advanced_testcase {
         // Check that the user is still unsubscribed from the forum.
         $this->assertFalse(\mod_forum\subscriptions::is_subscribed($author->id, $forum));
 
-        // Check the deprecated function too.
-        $this->assertFalse(forum_is_subscribed($author->id, $forum));
-        $this->assertEquals(1, count($this->getDebuggingMessages()));
-        $this->resetDebugging();
-
         // And is unsubscribed from the discussion again.
         $this->assertFalse(\mod_forum\subscriptions::is_subscribed($author->id, $forum, $discussion->id));
 
@@ -866,11 +721,6 @@ class mod_forum_subscriptions_testcase extends advanced_testcase {
         // Check that the user is still unsubscribed from the forum.
         $this->assertFalse(\mod_forum\subscriptions::is_subscribed($author->id, $forum));
 
-        // Check the deprecated function too.
-        $this->assertFalse(forum_is_subscribed($author->id, $forum));
-        $this->assertEquals(1, count($this->getDebuggingMessages()));
-        $this->resetDebugging();
-
         // And is subscribed to the discussion again.
         $this->assertTrue(\mod_forum\subscriptions::is_subscribed($author->id, $forum, $discussion->id));
 
@@ -886,11 +736,6 @@ class mod_forum_subscriptions_testcase extends advanced_testcase {
         // Check that the user is still unsubscribed from the forum.
         $this->assertFalse(\mod_forum\subscriptions::is_subscribed($author->id, $forum));
 
-        // Check the deprecated function too.
-        $this->assertFalse(forum_is_subscribed($author->id, $forum));
-        $this->assertEquals(1, count($this->getDebuggingMessages()));
-        $this->resetDebugging();
-
         // But unsubscribed from the discussion.
         $this->assertFalse(\mod_forum\subscriptions::is_subscribed($author->id, $forum, $discussion->id));
 
@@ -901,46 +746,6 @@ class mod_forum_subscriptions_testcase extends advanced_testcase {
         )));
     }
 
-    /**
-     * Test that the deprecated forum_is_subscribed accepts numeric forum IDs.
-     */
-    public function test_forum_is_subscribed_numeric() {
-        global $DB;
-
-        $this->resetAfterTest(true);
-
-        // Create a course, with a forum.
-        $course = $this->getDataGenerator()->create_course();
-
-        $options = array('course' => $course->id, 'forcesubscribe' => FORUM_CHOOSESUBSCRIBE);
-        $forum = $this->getDataGenerator()->create_module('forum', $options);
-
-        // Create a user enrolled in the course as a students.
-        list($author) = $this->helper_create_users($course, 1);
-
-        // Check that the user is currently unsubscribed to the forum.
-        $this->assertFalse(forum_is_subscribed($author->id, $forum->id));
-        $this->assertEquals(1, count($this->getDebuggingMessages()));
-        $this->resetDebugging();
-
-        // It should match the result of when it's called with the forum object.
-        $this->assertFalse(forum_is_subscribed($author->id, $forum));
-        $this->assertEquals(1, count($this->getDebuggingMessages()));
-        $this->resetDebugging();
-
-        // And when the user is subscribed, we should also get the correct result.
-        \mod_forum\subscriptions::subscribe_user($author->id, $forum);
-
-        $this->assertTrue(forum_is_subscribed($author->id, $forum->id));
-        $this->assertEquals(1, count($this->getDebuggingMessages()));
-        $this->resetDebugging();
-
-        // It should match the result of when it's called with the forum object.
-        $this->assertTrue(forum_is_subscribed($author->id, $forum));
-        $this->assertEquals(1, count($this->getDebuggingMessages()));
-        $this->resetDebugging();
-    }
-
     /**
      * Test that the correct users are returned when fetching subscribed users from a forum where users can choose to
      * subscribe and unsubscribe.
@@ -1100,7 +905,6 @@ class mod_forum_subscriptions_testcase extends advanced_testcase {
         $cm = get_coursemodule_from_instance('forum', $forum->id);
         $context = \context_module::instance($cm->id);
         assign_capability('mod/forum:allowforcesubscribe', CAP_PROHIBIT, $roleids['student'], $context);
-        $context->mark_dirty();
         $this->assertFalse(has_capability('mod/forum:allowforcesubscribe', $context, $user->id));
 
         // Check that the user is no longer subscribed to the forum.
@@ -1127,11 +931,11 @@ class mod_forum_subscriptions_testcase extends advanced_testcase {
         // Reset the subscription cache.
         \mod_forum\subscriptions::reset_forum_cache();
 
-        // Filling the subscription cache should only use a single query.
+        // Filling the subscription cache should use a query.
         $startcount = $DB->perf_get_reads();
         $this->assertNull(\mod_forum\subscriptions::fill_subscription_cache($forum->id));
         $postfillcount = $DB->perf_get_reads();
-        $this->assertEquals(1, $postfillcount - $startcount);
+        $this->assertNotEquals($postfillcount, $startcount);
 
         // Now fetch some subscriptions from that forum - these should use
         // the cache and not perform additional queries.
@@ -1203,7 +1007,7 @@ class mod_forum_subscriptions_testcase extends advanced_testcase {
         $result = \mod_forum\subscriptions::fill_subscription_cache_for_course($course->id, $user->id);
         $this->assertNull($result);
         $postfillcount = $DB->perf_get_reads();
-        $this->assertEquals(1, $postfillcount - $startcount);
+        $this->assertNotEquals($postfillcount, $startcount);
         $this->assertFalse(\mod_forum\subscriptions::fetch_subscription_cache($disallowforum->id, $user->id));
         $this->assertFalse(\mod_forum\subscriptions::fetch_subscription_cache($chooseforum->id, $user->id));
         $this->assertTrue(\mod_forum\subscriptions::fetch_subscription_cache($initialforum->id, $user->id));
@@ -1218,7 +1022,7 @@ class mod_forum_subscriptions_testcase extends advanced_testcase {
             $this->assertTrue(\mod_forum\subscriptions::fetch_subscription_cache($initialforum->id, $user->id));
         }
         $finalcount = $DB->perf_get_reads();
-        $this->assertEquals(count($users), $finalcount - $postfillcount);
+        $this->assertNotEquals($finalcount, $postfillcount);
     }
 
     /**
@@ -1271,7 +1075,7 @@ class mod_forum_subscriptions_testcase extends advanced_testcase {
         $startcount = $DB->perf_get_reads();
         $this->assertNull(\mod_forum\subscriptions::fill_discussion_subscription_cache($forum->id));
         $postfillcount = $DB->perf_get_reads();
-        $this->assertEquals(1, $postfillcount - $startcount);
+        $this->assertNotEquals($postfillcount, $startcount);
 
         // Now fetch some subscriptions from that forum - these should use
         // the cache and not perform additional queries.
@@ -1338,7 +1142,7 @@ class mod_forum_subscriptions_testcase extends advanced_testcase {
             $this->assertInternalType('array', $result);
         }
         $finalcount = $DB->perf_get_reads();
-        $this->assertEquals(20, $finalcount - $startcount);
+        $this->assertNotEquals($finalcount, $startcount);
     }
 
     /**
@@ -1478,4 +1282,144 @@ class mod_forum_subscriptions_testcase extends advanced_testcase {
         $this->assertGreaterThan($suppliedcmcount, $calculatedcmcount);
     }
 
+    public function is_subscribable_forums() {
+        return [
+            [
+                'forcesubscribe' => FORUM_DISALLOWSUBSCRIBE,
+            ],
+            [
+                'forcesubscribe' => FORUM_CHOOSESUBSCRIBE,
+            ],
+            [
+                'forcesubscribe' => FORUM_INITIALSUBSCRIBE,
+            ],
+            [
+                'forcesubscribe' => FORUM_FORCESUBSCRIBE,
+            ],
+        ];
+    }
+
+    public function is_subscribable_provider() {
+        $data = [];
+        foreach ($this->is_subscribable_forums() as $forum) {
+            $data[] = [$forum];
+        }
+
+        return $data;
+    }
+
+    /**
+     * @dataProvider is_subscribable_provider
+     */
+    public function test_is_subscribable_logged_out($options) {
+        $this->resetAfterTest(true);
+
+        // Create a course, with a forum.
+        $course = $this->getDataGenerator()->create_course();
+        $options['course'] = $course->id;
+        $forum = $this->getDataGenerator()->create_module('forum', $options);
+
+        $this->assertFalse(\mod_forum\subscriptions::is_subscribable($forum));
+    }
+
+    /**
+     * @dataProvider is_subscribable_provider
+     */
+    public function test_is_subscribable_is_guest($options) {
+        global $DB;
+        $this->resetAfterTest(true);
+
+        $guest = $DB->get_record('user', array('username'=>'guest'));
+        $this->setUser($guest);
+
+        // Create a course, with a forum.
+        $course = $this->getDataGenerator()->create_course();
+        $options['course'] = $course->id;
+        $forum = $this->getDataGenerator()->create_module('forum', $options);
+
+        $this->assertFalse(\mod_forum\subscriptions::is_subscribable($forum));
+    }
+
+    public function is_subscribable_loggedin_provider() {
+        return [
+            [
+                ['forcesubscribe' => FORUM_DISALLOWSUBSCRIBE],
+                false,
+            ],
+            [
+                ['forcesubscribe' => FORUM_CHOOSESUBSCRIBE],
+                true,
+            ],
+            [
+                ['forcesubscribe' => FORUM_INITIALSUBSCRIBE],
+                true,
+            ],
+            [
+                ['forcesubscribe' => FORUM_FORCESUBSCRIBE],
+                false,
+            ],
+        ];
+    }
+
+    /**
+     * @dataProvider is_subscribable_loggedin_provider
+     */
+    public function test_is_subscribable_loggedin($options, $expect) {
+        $this->resetAfterTest(true);
+
+        // Create a course, with a forum.
+        $course = $this->getDataGenerator()->create_course();
+        $options['course'] = $course->id;
+        $forum = $this->getDataGenerator()->create_module('forum', $options);
+
+        $user = $this->getDataGenerator()->create_user();
+        $this->getDataGenerator()->enrol_user($user->id, $course->id);
+        $this->setUser($user);
+
+        $this->assertEquals($expect, \mod_forum\subscriptions::is_subscribable($forum));
+    }
+
+    public function test_get_user_default_subscription() {
+        global $DB;
+        $this->resetAfterTest(true);
+
+        // Create a course, with a forum.
+        $course = $this->getDataGenerator()->create_course();
+        $context = \context_course::instance($course->id);
+        $options['course'] = $course->id;
+        $forum = $this->getDataGenerator()->create_module('forum', $options);
+        $cm = get_coursemodule_from_instance("forum", $forum->id, $course->id);
+
+        // Create a user enrolled in the course as a student.
+        list($author, $student) = $this->helper_create_users($course, 2, 'student');
+        // Post a discussion to the forum.
+        list($discussion, $post) = $this->helper_post_to_forum($forum, $author);
+
+        // A guest user.
+        $this->setUser(0);
+        $this->assertFalse((boolean)\mod_forum\subscriptions::get_user_default_subscription($forum, $context, $cm, $discussion->id));
+        $this->assertFalse((boolean)\mod_forum\subscriptions::get_user_default_subscription($forum, $context, $cm, null));
+
+        // A user enrolled in the course.
+        $this->setUser($author->id);
+        $this->assertTrue((boolean)\mod_forum\subscriptions::get_user_default_subscription($forum, $context, $cm, $discussion->id));
+        $this->assertTrue((boolean)\mod_forum\subscriptions::get_user_default_subscription($forum, $context, $cm, null));
+
+        // Subscribption disabled.
+        $this->setUser($student->id);
+        \mod_forum\subscriptions::set_subscription_mode($forum->id, FORUM_DISALLOWSUBSCRIBE);
+        $forum = $DB->get_record('forum', array('id' => $forum->id));
+        $this->assertFalse((boolean)\mod_forum\subscriptions::get_user_default_subscription($forum, $context, $cm, $discussion->id));
+        $this->assertFalse((boolean)\mod_forum\subscriptions::get_user_default_subscription($forum, $context, $cm, null));
+
+        \mod_forum\subscriptions::set_subscription_mode($forum->id, FORUM_FORCESUBSCRIBE);
+        $forum = $DB->get_record('forum', array('id' => $forum->id));
+        $this->assertTrue((boolean)\mod_forum\subscriptions::get_user_default_subscription($forum, $context, $cm, $discussion->id));
+        $this->assertTrue((boolean)\mod_forum\subscriptions::get_user_default_subscription($forum, $context, $cm, null));
+
+        // Admin user.
+        $this->setAdminUser();
+        $this->assertTrue((boolean)\mod_forum\subscriptions::get_user_default_subscription($forum, $context, $cm, $discussion->id));
+        $this->assertTrue((boolean)\mod_forum\subscriptions::get_user_default_subscription($forum, $context, $cm, null));
+    }
 }
