@@ -25,5 +25,9 @@
 
 defined('MOODLE_INTERNAL') || die();
 
-$plugin->version  = 2011051200;
-$plugin->requires = 2011060313;
+$plugin->component = 'qtype_numerical';
+$plugin->version   = 2019052000;
+
+$plugin->requires  = 2019051100;
+
+$plugin->maturity  = MATURITY_STABLE;
